import os

from magma_ven.utils import check_directory


class MagmaVen:
    """MAGMA Volcanic Eruption Notice (VEN) base class.

    Attributes:
        volcano_code (str): MAGMA Indonesia volcano code. 3 characters.
        start_date (str): Start date of VEN.
        end_date (str): End date of VEN.
        current_dir (str): Current directory. Defaults to current directory.
        verbose (bool): Verbose mode. Defaults to False.
        debug (bool): Debug mode. Defaults to False.
    """

    def __init__(
        self,
        volcano_code: str,
        start_date: str,
        end_date: str,
        current_dir: str = None,
        verbose: bool = False,
        debug: bool = False,
    ):
        """Init MAGMA Volcanic Eruption Notice (VEN) base class.

        Args:
            volcano_code (str): MAGMA volcano code.
            start_date (str): Start date of VEN.
            end_date (str): End date of VEN.
            current_dir (str): Current directory. Defaults to current directory.
            verbose (bool): Verbose mode. Defaults to False.
            debug (bool): Debug mode. Defaults to False.
        """
        self.volcano_code = volcano_code.upper()
        self.start_date = start_date
        self.end_date = end_date
        self.verbose = verbose
        self.debug = debug

        if current_dir is None:
            current_dir = os.getcwd()
        self.current_dir = current_dir or os.getcwd()
        self.output_dir, self.figures_dir, self.magma_dir = check_directory(current_dir)
        self.ven_dir = os.path.join(self.magma_dir, "ven")
        self.filename = f"{self.volcano_code}_{self.start_date}_{self.end_date}"



import datetime
import json
import os
import re
from typing import Tuple

import numpy as np


def check_directory(current_dir: str) -> Tuple[str, str, str]:
    """Checking existing directory

    Args:
        current_dir (str): Current directory

    Returns:
        Tuple[str, str, str]: Directory name, directory path, and file path
    """
    output_dir = os.path.join(current_dir, "output")
    os.makedirs(output_dir, exist_ok=True)

    figures_dir = os.path.join(output_dir, "figures")
    os.makedirs(figures_dir, exist_ok=True)

    magma_dir = os.path.join(output_dir, "magma")
    os.makedirs(magma_dir, exist_ok=True)

    return output_dir, figures_dir, magma_dir


def save(filename: str, response: dict) -> str:
    """Save response to file

    Args:
        filename (str): Filename
        response (dict): Response

    Returns:
        str: Saved response location
    """
    with open(filename, "w", encoding="utf-8") as f:
        json.dump(response, f, ensure_ascii=False, indent=4)

    return filename


def activity_level(level: str) -> int:
    """Transform activity text into level.

    Args:
        level (str): Level Activity text. Eg: Level I (Normal), Level II (Waspada)

    Returns:
        int: Level
    """
    if "IV" in level:
        return 4
    if "III" in level:
        return 3
    if "II" in level:
        return 2
    return 1


def translate_ash_color(color: str) -> str:
    """Translate ash color

    Args:
        color (str): Color

    Returns:
        color (str): Translated color
    """
    color = (
        color.replace("putih", "white")
        .replace("kelabu", "gray")
        .replace("coklat", "brown")
        .replace("hitam", "black")
    )

    return color


def translate_intensity(intensity: str) -> str:
    """Translate ash intensity

    Args:
        intensity (str): Intensity

    Returns:
        str: Translated intensity
    """
    intensity = (
        intensity.replace("tipis", "thin")
        .replace("sedang", "medium")
        .replace("tebal", "thick")
    )

    return intensity


def translate_wind_direction(wind_direction: str) -> str:
    """Translate wind direction

    Args:
        wind_direction (str): Wind direction

    Returns:
        str: Translated wind direction
    """
    wind_direction = (
        wind_direction.replace("utara", "north")
        .replace("timur laut", "northeast")
        .replace("barat daya", "southwest")
        .replace("barat laut", "northwest")
        .replace("timur", "east")
        .replace("tenggara", "southeast")
        .replace("selatan", "south")
        .replace("barat", "west")
    )

    return wind_direction


def ash_directions_text(directions: str) -> str:
    """Transform ash directions

    Args:
        directions (str): Ash directions separated by ", "

    Returns:
        str: Ash directions text
    """
    directions = directions.split(", ")

    if len(directions) == 1:
        return directions[0]

    if len(directions) == 2:
        return directions[0] + " and " + directions[1]

    length = len(directions)

    _directions = directions[0]
    for i in range(length):
        if i == 0:
            continue

        if i == length - 1:
            _directions = f"{_directions}, and {directions[i]}"

        _directions = f"{_directions}, {directions[i]}"

    return _directions


def translate_visual_description(
    volcano_name: str,
    volcano_height: int | float,
    iso_datetime: str,
    extracted_description: dict,
) -> str:
    """Translate visual description

    Args:
        volcano_name (str): Volcano name
        iso_datetime (str): Datetime of eruption
        volcano_height (int): Volcano height in meter
        extracted_description (dict): Extracted description including
                    'column_height','ash_color','ash_intensity','ash_direction'

    Returns:
        str: Translated visual description in english
    """
    column_height = extracted_description["column_height"]

    local_datetime = datetime.datetime.fromisoformat(iso_datetime)
    local_datetime_str = local_datetime.strftime("%A, %B %d, %Y, at %H:%M %Z")

    column_height_above_sea_level = (
        column_height + volcano_height if column_height > 0 else None
    )

    ash_observed = (
        (
            f"with an ash column observed reaching {column_height} meters above the summit "
            f"({column_height_above_sea_level} meters above sea level)"
        )
        if column_height > 0
        else "the eruption was not visually observed."
    )

    if column_height is np.nan:
        return f"An eruption of Mt. {volcano_name} occurred on {local_datetime_str}, {ash_observed}."

    ash_colors = extracted_description["ash_color"].split(", ")
    ash_colors_text = (
        ash_colors[0] if len(ash_colors) == 1 else f"{ash_colors[0]} to {ash_colors[1]}"
    )

    ash_intensities = extracted_description["ash_intensity"].split(", ")
    ash_intensities_text = (
        ash_intensities[0]
        if len(ash_intensities) == 1
        else f"{ash_intensities[0]} to {ash_intensities[1]}"
    )

    ash_direction = ash_directions_text(extracted_description["ash_direction"])
    ash_information = (
        f"The ash column was observed to be {ash_colors_text} in color, "
        f"with {ash_intensities_text} intensity, heading toward the {ash_direction}"
    )

    description = f"An eruption of Mt. {volcano_name} occurred on {local_datetime_str}, {ash_observed}. {ash_information}."

    return description


def extract_eruption_data(text: str, locale: str = "id") -> dict:
    """Extract eruption data

    Args:
        text (str): Text to extract
        locale (str, optional): Locale. Defaults to "id".

    Returns:
        dict: Extracted data
    """
    locale = locale.lower()

    if "Visual letusan tidak teramati" in text:
        return {
            "column_height": np.nan,
            "ash_color": None,
            "ash_intensity": None,
            "ash_direction": None,
        }

    text = text.replace("&plusmn; ", "")

    column_height = re.search(r"tinggi kolom abu teramati (\d+)\s*m", text)
    column_height = float(column_height.group(1)) if column_height else np.nan

    ash_color = re.search(r"berwarna ([\w\s]+?) dengan", text)
    ash_intensity = re.search(r"dengan intensitas (\w+)", text)
    ash_direction = re.search(r"ke arah ([\w\s]+)[.,]", text)

    if ash_color is not None:
        ash_color = ash_color.group(1).replace(" hingga ", ", ")
        ash_color = translate_ash_color(ash_color) if locale == "en" else ash_color

    if ash_intensity is not None:
        ash_intensity = ash_intensity.group(1).replace(" hingga ", ", ")
        ash_intensity = (
            translate_intensity(ash_intensity) if locale == "en" else ash_intensity
        )

    if ash_direction is not None:
        ash_direction = ash_direction.group(1).replace(" dan ", ", ")
        ash_direction = (
            translate_wind_direction(ash_direction) if locale == "en" else ash_direction
        )

    data = {
        "column_height": column_height,
        "ash_color": ash_color,
        "ash_intensity": ash_intensity,
        "ash_direction": ash_direction,
    }

    return data


def extract_instrument_data(text: str, locale: str = "id") -> dict:
    """Extract instrument data

    Args:
        text (str): Text to extract
        locale (str, optional): Locale. Defaults to "id".

    Returns:
        dict: Extracted data
    """
    locale = locale.lower()

    amplitude = re.search(r"amplitudo maksimum (\d+)\s*mm", text)
    amplitude = float(amplitude.group(1)) if amplitude else np.nan

    duration = re.search(r"durasi (\d+)\s*detik", text)
    duration = float(duration.group(1)) if duration else np.nan

    if locale == "en":
        text = (
            f"This eruption was recorded on a seismograph with a maximum amplitude of "
            f"{amplitude} mm and a duration of {duration} seconds."
        )

    data = {
        "amplitude": amplitude,
        "duration": duration,
        "description": text if amplitude > 0.0 else None,
    }

    return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pkg_resources import get_distribution

from magma_ven.download import Download
from magma_ven.magma_ven import MagmaVen

__version__ = get_distribution("magma-ven").version
__author__ = "Martanto"
__author_email__ = "martanto@live.com"
__license__ = "MIT"
__copyright__ = "Copyright (c) 2024, MAGMA Indonesia"
__url__ = "https://github.com/martanto/magma-ven"

__all__ = [
    "__version__",
    "__author__",
    "__author_email__",
    "__license__",
    "__copyright__",
    "MagmaVen",
    "Download",
]



import json
import os
from typing import Self

import pandas as pd
import requests
from pandas.errors import EmptyDataError

import magma_ven
from magma_ven.const import URL_FILTER
from magma_ven.magma_ven import MagmaVen
from magma_ven.utils import (
    activity_level,
    extract_eruption_data,
    extract_instrument_data,
    save,
    translate_visual_description,
)


class Download(MagmaVen):
    def __init__(
        self,
        token: str,
        volcano_code: str,
        start_date: str,
        end_date: str,
        current_dir: str = None,
        locale: str = "id",
        url_filter: str = None,
        verbose: bool = False,
        debug: bool = False,
    ):
        super().__init__(
            volcano_code, start_date, end_date, current_dir, verbose, debug
        )

        self.dates = f"{start_date}_{end_date}"

        self.token = token
        self.headers: dict[str, str] = {}
        self.json_dir = os.path.join(self.ven_dir, "json")
        self.page_dir = os.path.join(self.json_dir, "pages", volcano_code, self.dates)
        self.daily_dir = os.path.join(self.json_dir, "daily", volcano_code)
        self.files: list[str] = []
        self.locale = locale
        self.url_filter = url_filter or URL_FILTER

        self.excel_dir = os.path.join(self.ven_dir, "excel", volcano_code, self.dates)
        self.csv_dir = os.path.join(self.ven_dir, "csv", volcano_code, self.dates)
        self.data = []

        print(f"Version: {magma_ven.__version__}")

    @property
    def df(self) -> pd.DataFrame:
        assert len(self.files) > 1, FileNotFoundError(
            f"❌ JSON per pages not found. Please download first."
        )

        os.makedirs(self.daily_dir, exist_ok=True)

        for file in self.files:
            response = json.load(open(file))
            for _data in response["data"]:
                data = {}
                extracted = extract_eruption_data(
                    _data["deskripsi"]["visual"], locale=self.locale
                )

                extracted_instrument = extract_instrument_data(
                    _data["deskripsi"]["instrumental"], locale=self.locale
                )

                column_height = extracted["column_height"]

                visual_description = _data["deskripsi"]["visual"]

                visual_description = (
                    translate_visual_description(
                        volcano_name=_data["nama_gunung_api"],
                        volcano_height=_data["elevation"],
                        iso_datetime=_data["iso_datetime"],
                        extracted_description=extracted,
                    )
                    if self.locale == "en"
                    else visual_description
                )

                data["ven_id"] = _data["id"]
                data["volcano_code"] = _data["code_ga"]
                data["volcano_name"] = _data["nama_gunung_api"]
                data["local_datetime"] = _data["local_datetime"]
                data["local_time_zone"] = _data["time_zone"]
                data["iso_datetime"] = _data["iso_datetime"]
                data["level"] = activity_level(_data["tingkat_aktivitas"])
                data["visually_observed"] = column_height > 0
                data["continuing_eruption"] = extracted_instrument["amplitude"] == 0
                data["column_height"] = column_height
                data["column_height_above_sea_level"] = (
                    column_height + _data["elevation"]
                )
                data["ash_color"] = extracted["ash_color"]
                data["ash_intensity"] = extracted["ash_intensity"]
                data["ash_direction"] = extracted["ash_direction"]
                data["max_amplitude_mm"] = extracted_instrument["amplitude"]
                data["duration_second"] = extracted_instrument["duration"]
                data["photo"] = _data["foto"] if column_height > 0 else None
                data[f"eruption_description_{self.locale}"] = visual_description
                data[f"instrument_description_{self.locale}"] = extracted_instrument[
                    "description"
                ]
                data["recommendation_id"] = _data["rekomendasi"]
                data["reported_by"] = _data["pelapor"]
                data["source"] = _data["share"]["url"]

                self.data.append(data)

        return pd.DataFrame(self.data)

    def download(self, params: dict = None) -> dict:
        """Download Volcanic Eruption Notice (VEN).

        Args:
            params (dict): code, start_date, and end_date.

        Returns:
            dict
        """

        if len(self.headers.keys()) == 0:
            raise ValueError(f"❌ Headers not found.")

        payload: str = json.dumps(
            {
                "code": self.volcano_code.upper(),
                "start_date": self.start_date,
                "end_date": self.end_date,
            }
        )

        try:
            response = requests.request(
                "GET",
                self.url_filter,
                headers=self.headers,
                data=payload,
                params=params,
            )

            success = response.ok
            response = response.json()

            if not success or "errors" in response.keys():
                raise ValueError(f'❌ Download Error :: {response["errors"]}')

            keys = ["data", "links", "meta"]

            for key in keys:
                if key not in response.keys():
                    raise KeyError(f"❌ '{key}' not found in response :: {response}")

            if len(response["data"]) == 0:
                raise ValueError(
                    f"❌ No data found for {self.volcano_code} :: {response}"
                )

            return response
        except Exception as e:
            raise ValueError(f"❌ Failed to download JSON :: {e}")

    def download_per_page(self, response: dict) -> list[str]:
        """Download Volcanic Eruption Notice (VEN) per page.

        Args:
            response (dict): Volcanic Eruption Notice response.

        Returns:
            list[str]
        """
        pages: list[str] = []
        last_page = response["meta"]["last_page"]

        if last_page == 1:
            pass

        if self.verbose:
            print(f"ℹ️ Downloading from page 2 to {last_page}")
            print("=" * 60)

        for page in range(2, last_page + 1):
            # code_startDate_endDate_page.json
            # example: AWU_2025-01-01_2025-01-31_1.json
            filename = f"{self.filename}_{page}.json"
            json_per_page = os.path.join(self.page_dir, filename)

            if os.path.exists(json_per_page):
                if self.verbose:
                    print(f"✅ Skip. JSON for page #{page} exists :: {json_per_page}")
                pages.append(json_per_page)
                continue

            response = self.download({"page": page})
            save(json_per_page, response)

            pages.append(json_per_page)

            if self.verbose:
                print(f"✅ JSON for page #{page} downloaded :: {json_per_page}")

        self.files = self.files + pages
        return pages

    def download_first_page(self, first_page_json: str = None) -> dict:
        """Download Volcanic Eruption Notice (VEN) first page.

        Args:
            first_page_json (str): First page JSON path.

        Returns:
            dict
        """
        os.makedirs(self.ven_dir, exist_ok=True)
        os.makedirs(self.json_dir, exist_ok=True)
        os.makedirs(self.page_dir, exist_ok=True)

        first_page_json = first_page_json or os.path.join(
            self.page_dir, f"{self.filename}_1.json"
        )

        if os.path.isfile(first_page_json):
            if self.verbose:
                print(f"✅ JSON First Page exists :: {first_page_json}")
            self.files.append(first_page_json)
            return json.load(open(first_page_json))

        if self.verbose:
            print(f"⌛ Downloading JSON First Page :: {first_page_json}")

        response = self.download()
        save(first_page_json, response)

        if self.verbose:
            total = response["meta"]["total"]
            print(f"ℹ️ Total Data :: {total}")
            print(f"✅ JSON First Page downloaded :: {first_page_json}")

        self.files.append(first_page_json)

        return response

    def ven(self, token: str = None) -> Self:
        """Get Volcanic Eruption Notice (VEN) API.

        Args:
            token (str, optional): MAGMA Token.

        Returns:
            MagmaVen
        """
        token = token or self.token
        self.headers = {
            "Authorization": "Bearer " + token,
            "Content-Type": "application/json",
        }

        response = self.download_first_page()
        self.download_per_page(response)

        if self.verbose:
            print("ℹ️ JSON Files :: ", len(self.files))

        return self

    def to_excel(self, output_path: str = None) -> str:
        """Convert Volcanic Eruption Notice (VEN) to Excel.

        Args:
            output_path (str, optional): Excel Output path.

        Returns:
            str: Excel Output path.
        """
        assert len(self.df) > 0, EmptyDataError(
            f"❌ Data not found. Try to redownload it once again"
        )

        if output_path is None:
            output_path = self.excel_dir
        os.makedirs(output_path, exist_ok=True)

        filename = f"{self.filename}.xlsx"
        filepath = os.path.join(output_path, filename)
        self.df.to_excel(filepath, index=False)

        return filepath

    def to_csv(self, output_path: str = None) -> str:
        """Convert Volcanic Eruption Notice (VEN) to Excel.

        Args:
            output_path (str, optional): CSV output path.

        Returns:
            str: CSV output path.
        """
        assert len(self.df) > 0, EmptyDataError(
            f"❌ Data not found. Try to redownload it once again"
        )

        if output_path is None:
            output_path = self.csv_dir
        os.makedirs(output_path, exist_ok=True)

        filename = f"{self.filename}.csv"
        filepath = os.path.join(output_path, filename)
        self.df.to_csv(filepath, index=False)

        return filepath
